/* -----------------------------------------------------------------------------------------------------
  SOFTWARE BASICO - fractal
   ---------------------------------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define VIDEORAM   ((byte *)0xE0000000)  // endereo virtual da RAM de vdeo
#define NLINHAS    480   // nmero de linhas na tela
#define NCOLUNAS   640   // nmero de colunas na tela

/*** struct and type declarations ***/

typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned int dword;

/*** declaraes de referncias externas e internas ***/

extern void cadastrar(int matricula);
extern void inicializar(int modo);  // vdeo grfico


void desenhar_fractal() {
/*
 Calculation of Julia set
 ------------------------
 Written by Kiselev Y. 1994

 Definitions of complex numbers:
   z = x + i*y  screen coordinates   xini <= x < xini+1024*xstep
   u = s + i*t  parameter         \  yini <= y < yini+768*ystep
 Equation used:
   f(z) = z^2 - u
 Iteration:
   f(z), f(f(z)), f(f(f(z))), ...
   until |f(...)| > 2  or  max_it iterations
 Result:
   pixel value at (x,y) is the number of iterations.
*/
   float xini,yini,x,x2,y,y2,xy;
   byte *ypix;  // apontador para a linha de video
   dword xpix;  // posio dentro da linha de video
   byte pix;    // valor do pixel

   yini=-1.15;
   for(ypix=VIDEORAM; ypix<VIDEORAM+NLINHAS*NCOLUNAS; ypix+=NCOLUNAS) {
     xini=-1.55;
     for(xpix=0; xpix<NCOLUNAS; xpix++) {
       x=xini;
       y=yini;
       for(pix=0; pix<255; pix++) {  // max_it=255
         x2=x*x;
         y2=y*y;
         if(x2+y2>4.0) break;  // |z|=sqrt(x^2+y^2) < 2
         xy = x*y;
         x = x2-y2-0.76;  // -s
         y = xy+xy-0.09;  // -t
       }
       ypix[xpix] = pix;
       xini+=0.005;   // +xstep
     }
     yini+=0.005;     // +ystep
   }
}

void main() {
   int c;
   cadastrar(20012345);
   inicializar(1);
   desenhar_fractal();
   c=getchar();
}
