# gawk -f us.awk usa.asm
BEGIN {
    # prepare to make two more, i.e. three passes
    ARGC+=2
    ARGV[ARGC-2] = ARGV[ARGC-3]
    ARGV[ARGC-1] = ARGV[ARGC-3]
    # pass 1: get header values (i.e. number of variables)
    # pass 2: write constant section and variable defines
    # pass 3: write code
}
# GROUP statements marks start of file
/GROUP DGROUP/ {
  pass++
  if(pass==1) {
     # determine file name for output file
     split(FILENAME, fnameparts, ".")
     outfilename = fnameparts[1] ".osm" }
  if(pass==2) {
    # varaddr is known after first pass
    print "  dd data_ende ; data size" > outfilename
    print "  dd code_ende-data_ende ; code size"  > outfilename
    print "  dd " varaddr " ; variable size" > outfilename
    print "  dd 120 ; stack size" > outfilename
    print ""  > outfilename
    print "bits 32" > outfilename
    print "org -16" > outfilename
    print "" > outfilename }
  if(pass==3) {
    print "data_ende:" > outfilename }
  varaddr=0
  # for peephole optimization
  hold[1] = ""
  hold[2] = ""
  hold[3] = ""
  hold[4] = ""
}
/SECTION/ {
  # stop printing at following SECTION
  if(string==1) { string++ }
  if(const==1)  { const--; }
  if(code==1)   { code++;  print "code_ende:" > outfilename }
  if(data==1)   { data++ }
}
# The following 3 patterns are for peephole optimization
# of calls to constant far pointers, for example for calling
# call gate used for system calls.
/MOV	EAX,/ {
  # example: MOV EAX,00H
  split($2,a,",")
  if(substr(a[2], length(a[2]), 1)=="H") {
     offset = a[2]
     hold[1] = $0
     $0 = ""
  }
}
/MOV	DWORD \[SS:EBP+/ {
  split($3,a,",")
  split(a[1],b,"+")
  if(substr(b[2], length(b[2]), 1)=="]") { b[2] = substr(b[2], 1, length(b[2])-1) }
  if(substr(b[2], length(b[2]), 1)=="H") {
     if(b[3]=="04H]" && hold[2]=="next") {
        # example: MOV DWORD [SS:EBP+0FFFFFFD4H+04H],01000H
        ebp = b[2]
        selector = a[2]
        hold[2] = $0
        $0 = ""
     }
     if(b[2]==ebp && a[2]=="EAX" && hold[3]=="next") {
        # example: MOV DWORD [SS:EBP+0FFFFFFD4H],EAX
        hold[3] = $0
        $0 = ""
     }
  }
}
/CALL	DWORD FAR \[SS:EBP+/ {
  # example: CALL DWORD FAR [SS:EBP+0FFFFFFD4H]
  split($4,b,"+")
  if(substr(b[2], length(b[2]), 1)=="]") {
     if(substr(b[2], 1, length(b[2])-1)==ebp && hold[4]=="next") {
       $0 = "	CALL	" selector ":" offset
       hold[1] = ""
       hold[2] = ""
       hold[3] = ""
       hold[4] = ""
     }
  }
}
# print lines in data and code sections
{ if(string==1 || data==1 || code==1) {
    if($2=="SHORT") $2="NEAR"

    # for peephole optimization
    if(hold[2]=="next") {
       print hold[1] > outfilename
       hold[1] = ""
       hold[2] = ""
       hold[3] = ""
       hold[4] = ""
    }
    if(hold[3]=="next") {
       print hold[1] > outfilename
       print hold[2] > outfilename
       hold[1] = ""
       hold[2] = ""
       hold[3] = ""
       hold[4] = ""
    }
    if(hold[4]=="next") {
       print hold[1] > outfilename
       print hold[2] > outfilename
       print hold[3] > outfilename
       hold[1] = ""
       hold[2] = ""
       hold[3] = ""
       hold[4] = ""
    }

    if($0) { print $0 > outfilename }

    if(hold[3]!="") { hold[4] = "next" } else {
       if(hold[2]!="") { hold[3] = "next" } else {
          if(hold[1]!="") { hold[2] = "next" }
       }
    }

  }
  else if(const==1) {
    if(substr($1, length($1), 1)==":") {
      print "ES_" $1 > outfilename
      $1 = substr($1, 1, length($1)-1)
      print "%define " $1 " ES:ES_" $1 > outfilename
    }
    else if($1!="TIMES" || $2!="$$-$") {
      if($0) { print $0 > outfilename }
    }
  }
}
/RESB/ {
  # Define variables in pass two, after the header
  # has been written.
  if(pass==2 && $2=="RESB") { print "%define " $1 " " varaddr > outfilename }
  if($2=="RESB") varaddr+=$3
  if($1=="RESB") varaddr+=$2
}
# start printing data in pass 2
/SECTION _DATA/   { if(pass==2) data++; }
/SECTION _CONST/  { if(pass==2) const++; }
/SECTION _STRING/ { if(pass==2) string++; }
# start printing code in pass 3
/SECTION _TEXT/   { if(pass==3) code++; }

